/*
 * Copyright 2016 The jdeb developers.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.vafer.jdeb.utils;

import java.io.File;
import java.io.IOException;

/**
 * Simple utilities to deal with symbolic links
 */
public final class SymlinkUtils {
    private SymlinkUtils() {
    }

    public static boolean isSymbolicLink(final File file) throws IOException {
        final File canon;
        if (file.getParent() == null) {
            canon = file;
        } else {
            final File canonDir = file.getParentFile().getCanonicalFile();
            canon = new File(canonDir, file.getName());
        }
        return !canon.getCanonicalFile().equals(canon.getAbsoluteFile());
    }

    public static String readSymbolicLink(final File file) throws IOException {
        return file.getCanonicalFile().getPath();
    }

}
